@@ -2,14 +2,14 @@
 #
 #  Name:       RNetCDF-test.R
 #
-#  Version:    2.4-2
+#  Version:    2.5-2
 #
 #  Purpose:    Test functions to the NetCDF interface for R.
 #
 #  Author:     Pavel Michna (rnetcdf-devel@bluewin.ch)
 #              Milton Woods (miltonjwoods@gmail.com)
 #
-#  Copyright:  (C) 2010-2020 Pavel Michna, Milton Woods
+#  Copyright (C) 2004-2021 Pavel Michna and Milton Woods.
 #
 #===============================================================================#
 #
@@ -201,6 +201,14 @@ for (format in c("classic","offset64","data64","classic4","netcdf4")) {
       numtypes <- c(numtypes, "NC_INT64", "NC_UINT64")
       tally <- testfun(TRUE, TRUE, tally)
     }
+
+    inq_filter <- list()
+    inq_filter$filter_id <- c(2,1) # Shuffle, deflate
+    inq_filter$filter_params <- list(numeric(0),c(9))
+    var.def.nc(nc, "temp_filter", "NC_FLOAT", c("station", "time"),
+               chunking=TRUE, filter_id=inq_filter$filter_id,
+               filter_params=inq_filter$filter_params)
+    varcnt <- varcnt+1
   }
 
   for (numtype in numtypes) {
@@ -471,9 +479,12 @@ for (format in c("classic","offset64","data64","classic4","netcdf4")) {
     var.put.nc(nc, "rawdata_vector", rawdata[,,1])
     var.put.nc(nc, "snacks", snacks)
     var.put.nc(nc, "person", person)
+    tally <- testfun(TRUE, TRUE, tally)
     if (has_bit64) {
       var.put.nc(nc, "stationid", mybig64)
+      tally <- testfun(TRUE, TRUE, tally)
     }
+    var.put.nc(nc, "temp_filter", mytemperature)
     tally <- testfun(TRUE, TRUE, tally)
   }
 
@@ -570,8 +581,17 @@ for (format in c("classic","offset64","data64","classic4","netcdf4")) {
       cat("Feature not available in this NetCDF library version.\n")
     } else {
       y <- 0.5
+      tally <- testfun(x,y,tally)
+    }
+
+    # Check multi-filter inquiry:
+    cat("Check filter settings after writing temp_filter ...")
+    x <- var.inq.nc(nc, "temp_filter")
+    if (is.null(x$filter_id) && is.null(x$filter_params)) {
+      cat("Multi-filters not available in this NetCDF library version.\n")
+    } else {
+      tally <- testfun(x[names(inq_filter)], inq_filter, tally)
     }
-    tally <- testfun(x,y,tally)
   }
 
 #  sync.nc(nc)
